export function lazyJacobian(f) {
    const h = 0.01;
    return (x, y, z) => [
        (f(x + h, y, z) - f(x, y, z)) / h,
        (f(x, y + h, z) - f(x, y, z)) / h,
        (f(x, y, z + h) - f(x, y, z)) / h
    ];
}
export function intersectSurfacesParametric() {
}
export function intersectSurfacesImplicit(f, g, df, dg, q0, samples, step) {
    const points = [];
    let p = q0;
    while (points.length < samples) {
        const tangent = new THREE.Vector3().crossVectors(new THREE.Vector3().fromArray(df(p.x, p.y, p.z)), new THREE.Vector3().fromArray(dg(p.x, p.y, p.z))).normalize();
        points.push(curvePoint(f, g, df, dg, p.clone().add(tangent.multiplyScalar(step))));
        p = points[points.length - 1];
    }
    return points;
}
export function curvePoint(f, g, df, dg, q0) {
    const epsilon = 0.1;
    let qk = q0, diff = new THREE.Vector3();
    do {
        const dfqk = df(qk.x, qk.y, qk.z), dgqk = dg(qk.x, qk.y, qk.z), jacT = [dfqk, dgqk];
        const matrix = [[0, 0], [0, 0]];
        for (let i = 0; i < 2; ++i)
            for (let j = 0; j < 2; ++j)
                for (let k = 0; k < 3; ++k) {
                    matrix[i][j] += jacT[i][k] * jacT[j][k];
                }
        const [alpha, beta] = solve22Matrix(matrix, [
            -f(qk.x, qk.y, qk.z),
            -g(qk.x, qk.y, qk.z)
        ]);
        diff =
            new THREE.Vector3().fromArray(dfqk).multiplyScalar(alpha).add(new THREE.Vector3().fromArray(dgqk).multiplyScalar(beta));
        qk = qk.clone().add(diff);
    } while (diff.length() >= epsilon);
    return qk;
}
export function solve22Matrix([[a, b], [c, d]], [y1, y2]) {
    return [
        (y1 * d - b * y2) / (a * d - b * c),
        (a * y2 - y1 * c) / (a * d - b * c)
    ];
}
